@@ -1,7 +1,7 @@
 /*
- * @(#) $Id: webindex.c 273 2015-07-05 06:21:32Z leres $ (XSE)
+ * @(#) $Id: webindex.c 282 2016-03-06 02:22:33Z leres $ (XSE)
  *
- * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016
  *	Craig Leres
  *
  * Redistribution and use in source and binary forms, with or without
@@ -16,7 +16,7 @@
  */
 #ifndef lint
 static const char copyright[] =
-    "@(#) Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015\nCraig Leres\n";
+    "@(#) Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016\nCraig Leres\n";
 #endif
 
 #include <sys/types.h>
@@ -108,6 +108,7 @@ int recurse;
 int verbose;
 int errors;
 int killed;				/* child was killed */
+char affix[2048];
 char prefix[2048];
 char *reason;
 
@@ -162,6 +163,7 @@ char *findsection(struct webfile *, struct cf *);
 void header(FILE *, struct cf *);
 void hookupthumbnails(struct cf *, struct webfile *, int);
 int invisible(const char *, struct cf *, int);
+int isdot(const char *);
 int isthumbfn(const char *);
 int main(int, char **);
 void parsethumbnail(struct cf *, struct webfile *);
@@ -184,7 +186,8 @@ int
 main(int argc, char **argv)
 {
 	char *cp;
-	int i, op;
+	const char *p;
+	int i, d, op;
 	mode_t nmode, omode;
 
 	if ((cp = strrchr(argv[0], '/')) != NULL)
@@ -227,10 +230,26 @@ main(int argc, char **argv)
 		(void)umask(nmode);
 	}
 
-	if (argc == optind)
+	if (argc == optind) {
+		affix[0] = '\0';
 		(void)process(".");
-	else for (i = optind; i < argc; ++i)
-		(void)process(argv[i]);
+	} else {
+		for (i = optind; i < argc; ++i) {
+			if (isdot(argv[i]))
+				affix[0] = '\0';
+			else {
+				/* Insure trailing slash */
+				strlcpy(affix, argv[i], sizeof(affix));
+				p = affix + strlen(affix) - 1;
+				if (p >= affix && *p != '/')
+					strlcat(affix, "/", sizeof(affix));
+			}
+			if (pushd(argv[i], NULL, &d)) {
+				(void)process(".");
+				(void)popd(NULL, d);
+			}
+		}
+	}
 
 	exit(errors != 0);
 }
@@ -241,7 +260,7 @@ process(const char *dname)
 {
 	const char *p, *indexfn;
 	const char *table, *tr, *tail;
-	char *cp, *section;
+	char *cp, *section, *prefixend;
 	int i, row, col, force, intable, savedtz;
 	time_t file_time, ind_time;
 	struct webfile *wp, *webfiles;
@@ -256,11 +275,11 @@ process(const char *dname)
 	DIRENTRY *dp;
 	struct stat sbuf;
 	char buf[128];
-	char dn[2048];
 	static char tz[] = "TZ";
 
 	if (verbose > 1)
-		fprintf(stderr, "%s: processing %s%s\n", prog, prefix, dname);
+		fprintf(stderr, "%s: processing %s%s%s\n",
+		    prog, affix, prefix, dname);
 
 	/* Initialization */
 	dirp = NULL;
@@ -282,17 +301,8 @@ process(const char *dname)
 	otz = NULL;
 	savedtz = 0;
 
-	if (strcmp(dname, ".") == 0)
-		dn[0] = '\0';
-	else {
-		strlcpy(dn, dname, sizeof(dn));
-		cp = dn + strlen(dn) - 1;
-		if (cp >= dn && *cp != '/')
-			strlcat(dn, "/", sizeof(dn));
-	}
-
 	/* Save our current working directory */
-	if (!pushd(dname, NULL, &d))
+	if (!pushd(dname, &prefixend, &d))
 		goto bail;
 
 	/* Read configuration info */
@@ -331,8 +341,8 @@ process(const char *dname)
 	/* Open the directory for scanning */
 	dirp = opendir(".");
 	if (dirp == NULL) {
-		fprintf(stderr, "%s: process: opendir %s%s: %s\n",
-		    prog, prefix, dn, strerror(errno));
+		fprintf(stderr, "%s: process: opendir %s: %s%s\n",
+		    prog, affix, prefix, strerror(errno));
 		++errors;
 		goto bail;
 	}
@@ -341,7 +351,7 @@ process(const char *dname)
 		/* OK if index.html doesn't exist */
 		if (errno != ENOENT) {
 			fprintf(stderr, "%s: process: lstat %s%s%s: %s\n",
-			    prog, prefix, dn, indexfn, strerror(errno));
+			    prog, affix, prefix, indexfn, strerror(errno));
 			++errors;
 			goto bail;
 		}
@@ -351,14 +361,14 @@ process(const char *dname)
 		/* If it doesn't look like can update the index file, bail */
 		if ((sbuf.st_mode & S_IFMT) == S_IFLNK) {
 			fprintf(stderr, "%s: process: %s%s%s: is a symlink\n",
-			    prog, prefix, dn, indexfn);
+			    prog, affix, prefix, indexfn);
 			++errors;
 			goto bail;
 		}
 		if ((sbuf.st_mode & S_IFMT) != S_IFREG) {
 			fprintf(stderr,
 			    "%s: process: %s%s%s: not a regular file\n",
-			    prog, prefix, dn, indexfn);
+			    prog, affix, prefix, indexfn);
 			++errors;
 			goto bail;
 		}
@@ -372,7 +382,7 @@ process(const char *dname)
 		f = fopen(indexfn, "r");
 		if (f == NULL) {
 			fprintf(stderr, "%s: process: fopen 1 %s%s%s: %s\n",
-			    prog, prefix, dn, indexfn, strerror(errno));
+			    prog, affix, prefix, indexfn, strerror(errno));
 			++errors;
 			goto bail;
 		}
@@ -383,7 +393,7 @@ process(const char *dname)
 		} else if (strstr(buf, magic) == NULL) {
 			fprintf(stderr,
 			    "%s: skipping %s%s%s (not created by %s)\n",
-			    prog, prefix, dn, indexfn, prog);
+			    prog, affix, prefix, indexfn, prog);
 			++errors;
 			goto bail;
 		}
@@ -418,7 +428,7 @@ process(const char *dname)
 
 		if (lstat(dp->d_name, &sbuf) < 0) {
 			fprintf(stderr, "%s: process: lstat %s%s%s: %s\n",
-			    prog, prefix, dn, dp->d_name, strerror(errno));
+			    prog, affix, prefix, dp->d_name, strerror(errno));
 			++errors;
 			/* XXX skip these */
 			continue;
@@ -432,7 +442,7 @@ process(const char *dname)
 			if (stat(dp->d_name, &sbuf) < 0) {
 				fprintf(stderr,
 				    "%s: process: stat %s%s%s: %s\n",
-				    prog, prefix, dn, dp->d_name,
+				    prog, affix, prefix, dp->d_name,
 				    strerror(errno));
 				++errors;
 				/* XXX skip these */
@@ -567,13 +577,13 @@ process(const char *dname)
 	if (verbose)
 		fprintf(stderr, "%s: %s %s%s%s%s\n",
 		    prog, ind_time == 0 ? "creating" : "updating",
-		    prefix, dn, indexfn, reason);
+		    affix, prefix, indexfn, reason);
 
 	/* Create directory if it doesn't exist */
 	if (access(naildir(cf), F_OK) < 0) {
 		if (verbose)
 			fprintf(stderr, "%s: creating %s%s%s/\n",
-			    prog, dname, prefix, naildir(cf));
+			    prog, affix, prefix, naildir(cf));
 		if (mkdir(naildir(cf), 0777) < 0) {
 			fprintf(stderr, "%s: process: mkdir %s: %s\n",
 			    prog, naildir(cf), strerror(errno));
@@ -584,7 +594,7 @@ process(const char *dname)
 	f = fopen(indexfn, "w");
 	if (f == NULL) {
 		fprintf(stderr, "%s: process: fopen 2 %s%s%s: %s\n",
-		    prog, prefix, dn, indexfn, strerror(errno));
+		    prog, affix, prefix, indexfn, strerror(errno));
 		++errors;
 		goto bail;
 	}
@@ -651,7 +661,7 @@ process(const char *dname)
 	/* Look for orphan sections */
 	for (sp = cf->c_section; sp != NULL; sp = sp->s_next)
 		fprintf(stderr, "%s: warning: %s%s%s: section not found\n",
-		    prog, prefix, dn, sp->s_file);
+		    prog, affix, prefix, sp->s_file);
 
 	/* Look for orphan list elements */
 	reportunused(cf->c_alttext, "unused alttext");
@@ -669,7 +679,7 @@ bail:
 	}
 
 	/* Restore our current working directory */
-	(void)popd(NULL, d);
+	(void)popd(prefixend, d);
 
 	/* Restore timezone */
 	if (savedtz) {
@@ -688,7 +698,7 @@ bail:
 	/* Close the index file */
 	if (f != NULL && fclose(f) == EOF) {
 		fprintf(stderr, "%s: process: fclose %s%s%s: %s\n",
-		    prog, prefix, dn, indexfn, strerror(errno));
+		    prog, affix, prefix, indexfn, strerror(errno));
 		++errors;
 	}
 
@@ -768,9 +778,10 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 		} else {
 			/* Create/update the thumbnail image */
 			if (verbose)
-				fprintf(stderr, "%s: %s %s%s\n", prog,
-				    wp2->w_tn_time == 0 ?
-				    "creating" : "updating", prefix, icon);
+				fprintf(stderr, "%s: %s %s%s%s\n",
+				    prog, wp2->w_tn_time == 0 ?
+				    "creating" : "updating",
+				    affix, prefix, icon);
 			if (createthumb(wp2, icon, cf)) {
 				w = wp2->w_tnw;
 				h = wp2->w_tnh;
@@ -865,6 +876,7 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 			else
 				sprintf(cp, "%dM", C2K(wp->w_kbytes));
 			cp += strlen(cp);
+			*cp = '\0';
 		}
 
 		if (wp->w_timestamp > 0) {
@@ -879,10 +891,12 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 				p = defdatefmt;
 			if (cp > buf)
 				*cp++ = ' ';
+			*cp = '\0';
 			(void)strftime(cp, sizeof(buf) - strlen(buf), p, tm);
 		} else if (wp->w_timestamp == 0 && cf->c_timestamps != NULL)
-			fprintf(stderr, "%s: warning: %s%s:"
-			    " missing timestamp\n", prog, prefix, wp->w_name);
+			fprintf(stderr, "%s: warning: %s%s%s:"
+			    " missing timestamp\n",
+			    prog, affix, prefix, wp->w_name);
 		cp = buf;
 		if (*cp != '\0')
 			fprintf(f, "%s<td><small>%s</small></td></tr>\n",
@@ -1060,8 +1074,8 @@ reapthumbnails(const char *dir, struct cf *cf,
 	if ((dirp = opendir(dir)) == NULL) {
 		if (errno != ENOENT) {
 			fprintf(stderr,
-			    "%s: reapthumbnails: opendir %s%s: %s\n",
-			    prog, prefix, dir, strerror(errno));
+			    "%s: reapthumbnails: opendir %s%s%s: %s\n",
+			    prog, affix, prefix, dir, strerror(errno));
 			++errors;
 		}
 		return (1);
@@ -1136,8 +1150,9 @@ reapthumbnails(const char *dir, struct cf *cf,
 
 		/* Ignore file (i.e. gripe but don't delete) */
 		if (ignorefile) {
-			fprintf(stderr, "%s: ignoring %s%s/%s (unknown file)\n",
-			    prog, prefix, dir, dp->d_name);
+			fprintf(stderr,
+			    "%s: ignoring %s%s%s/%s (unknown file)\n",
+			    prog, affix, prefix, dir, dp->d_name);
 			++force;
 			reason = " (ignore file)";
 			continue;
@@ -1146,11 +1161,12 @@ reapthumbnails(const char *dir, struct cf *cf,
 		/* Reap file */
 		sprintf(temp, "%s/%s", dir, dp->d_name);
 		if (verbose)
-			fprintf(stderr, "%s: deleting %s%s\n",
-			    prog, prefix, temp);
+			fprintf(stderr, "%s: deleting %s%s%s\n",
+			    prog, affix, prefix, temp);
 		if (unlink(temp) < 0)
-			fprintf(stderr, "%s: reapthumbnails: unlink %s%s: %s\n",
-			    prog, prefix, temp, strerror(errno));
+			fprintf(stderr,
+			    "%s: reapthumbnails: unlink %s%s%s: %s\n",
+			    prog, affix, prefix, temp, strerror(errno));
 		++force;
 		reason = " (reap file)";
 nextfile:
@@ -1291,7 +1307,7 @@ createthumb(struct webfile *wp, const char *fn, struct cf *cf)
 			fprintf(stderr,
 			    "%s: createthumb: \"%s ...\" failed: %s\n",
 			    prog, what, strerror(errno));
-		fprintf(stderr, "%s: deleting %s%s", prog, prefix, fn);
+		fprintf(stderr, "%s: deleting %s%s%s", prog, affix, prefix, fn);
 		if (remove(fn) < 0 && errno != ENOENT)
 			fprintf(stderr, " failed: %s", strerror(errno));
 		fprintf(stderr, "\n");
@@ -1302,13 +1318,13 @@ createthumb(struct webfile *wp, const char *fn, struct cf *cf)
 		wp->w_tnh = h;
 	} else {
 		fprintf(stderr,
-		    "%s: deleting %s%s (bad format thumbnail)\n",
-			prog, prefix, fn);
+		    "%s: deleting %s%s%s (bad format thumbnail)\n",
+			prog, affix, prefix, fn);
 			++errors;
 		errno = 0;
 		if (unlink(fn) < 0 && errno != ENOENT)
-			fprintf(stderr, "%s: createthumb: unlink %s%s: %s\n",
-			    prog, prefix, fn, strerror(errno));
+			fprintf(stderr, "%s: createthumb: unlink %s%s%s: %s\n",
+			    prog, affix, prefix, fn, strerror(errno));
 	}
 	return (1);
 }
@@ -1436,8 +1452,14 @@ checkmode(const char *fn, mode_t mode, int wtype)
 			cmode = 0111;
 	}
 	if ((mode & smode) != smode || (mode & cmode) != 0)
-		fprintf(stderr, "%s: warning: %s%s: bad permission (0%o)\n",
-		    prog, prefix, fn, mode & 0777);
+		fprintf(stderr, "%s: warning: %s%s%s: bad permission (0%o)\n",
+		    prog, affix, prefix, fn, mode & 0777);
+}
+
+int
+isdot(const char *dn)
+{
+	return (strcmp(".", dn) == 0 || strcmp("./", dn) == 0);
 }
 
 int
@@ -1596,11 +1618,11 @@ pushd(const char *dname, char **endp, int *dp)
 	if (endp != NULL)
 		*endp = prefix + strlen(prefix);
 
-	if (strcmp(".", dname) == 0 || strcmp("./", dname) == 0) {
+	if (isdot(dname)) {
 		*dp = -1;
 		if (debug > 1)
-			fprintf(stderr, "%s: pushd noop \"%s\" (%d)\n",
-			    prog, prefix, *dp);
+			fprintf(stderr, "%s: pushd noop \"%s%s\" (%d)\n",
+			    prog, affix, prefix, *dp);
 		return (1);
 	}
 
@@ -1608,16 +1630,16 @@ pushd(const char *dname, char **endp, int *dp)
 	*dp = open(".", O_RDONLY);
 	if (*dp < 0) {
 		fprintf(stderr,
-		    "%s: pushd: open: Can't open directory %s: %s\n",
-		    prog, prefix, strerror(errno));
+		    "%s: pushd: open: Can't open directory %s%s: %s\n",
+		    prog, affix, prefix, strerror(errno));
 		++errors;
 		return (0);
 	}
 
 	/* Change directory */
 	if (chdir(dname) < 0) {
-		fprintf(stderr, "%s: pushd: chdir %s: %s\n",
-		    prog, prefix, strerror(errno));
+		fprintf(stderr, "%s: pushd: chdir %s%s%s: %s\n",
+		    prog, affix, prefix, dname, strerror(errno));
 		(void)close(*dp);
 		*dp = -1;
 		++errors;
@@ -1635,7 +1657,8 @@ pushd(const char *dname, char **endp, int *dp)
 	}
 
 	if (debug > 1)
-		fprintf(stderr, "%s: pushd \"%s\" (%d)\n", prog, prefix, *dp);
+		fprintf(stderr, "%s: pushd \"%s%s\" (%d)\n",
+		    prog, affix, prefix, *dp);
 	return (1);
 }
 
@@ -1660,7 +1683,8 @@ popd(char *endp, int d)
 		(void)close(d);
 	}
 	if (debug > 1)
-		fprintf(stderr, "%s: popd \"%s\" (%d)\n", prog, prefix, d);
+		fprintf(stderr, "%s: popd \"%s%s\" (%d)\n",
+		    prog, affix, prefix, d);
 	return (ret);
 }
 
